var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
/**
 * Create sidebar leaf of sidebar group.
 * Sidebar element without path isn't clickable.
 */
function convertToTreeElement(page, key, path) {
    return {
        title: page.frontmatter.title || page.title,
        children: [],
        collapsable: false,
        path,
        key,
    };
}
/**
 * Find parent in tree structure.
 * Iterates over keys and on every step tries to find next subtree within children.
 * @param {{children: *[], collapsable: boolean, title: string, path: string, key: string}} tree
 * @param {string[]} keys
 * @returns {{children: *[], collapsable: boolean, title: string, path: string, key: string}}
 */
function findParent(tree, keys) {
    let parent = tree;
    // eslint-disable-next-line no-restricted-syntax
    for (const key of keys) {
        // eslint-disable-next-line no-restricted-syntax
        for (const subtree of parent.children) {
            if (subtree.key === key) {
                parent = subtree;
                break;
            }
        }
    }
    return parent;
}
const SIDEBARS = new Map();
const GenerateTreePlugin = (options, ctx) => ({
    name: 'vuepress-plugin-generate-tree',
    ready() {
        return __awaiter(this, void 0, void 0, function* () {
            let { pages } = ctx;
            /** @type Map<string, string> */
            const { locales } = options;
            const prefixes = Array.from(locales.values());
            pages = pages.sort((p1, p2) => {
                if (p1.path < p2.path)
                    return -1;
                if (p1.path > p2.path)
                    return 0;
                return 0;
            });
            prefixes.forEach((prefix) => {
                // Find a root page
                const rootPage = pages.find((page) => page.path === prefix);
                if (rootPage === undefined) {
                    throw new TypeError(`Your page tree does not have root page. It should be placed in directory as ${prefix}index.md or ${prefix}README.md`);
                }
                // Convert the root page to tree element
                const tree = convertToTreeElement(rootPage, '');
                let prefixPages = pages.filter((page) => page.path !== prefix // exclude root page
                    && page.path.startsWith(prefix));
                // special case for default prefix `/`: because other prefixes are substring
                // of `/` prefix we need explicitly exclude pages with other prefixes
                if (prefix === '/') {
                    const otherPrefixes = prefixes.filter((p) => p !== '/');
                    prefixPages = prefixPages.filter(
                    // each page shouldn't start with any other prefix
                    (page) => !otherPrefixes.some((otherPrefix) => page.path.startsWith(otherPrefix)));
                }
                // fill the tree with pages
                prefixPages.forEach((page) => {
                    // split path by `/` and remove empty parts
                    // /abc/ -> ["abc"]
                    // /abc/d.html -> ["abc", "d.html"]
                    const keys = page.path.split('/').filter((pathPart) => pathPart).filter((pathPart) => pathPart !== prefix);
                    const parent = findParent(tree, keys);
                    // eslint-disable-next-line no-underscore-dangle
                    if (page._strippedContent.trim()) {
                        // if page has content we should create clickable tree leaf
                        parent.children.push(convertToTreeElement(page, keys[keys.length - 1], page.path));
                    }
                    else {
                        parent.children.push(convertToTreeElement(page, keys[keys.length - 1].slice(0, -5)));
                    }
                });
                // add tree to mapping `prefix → tree`
                SIDEBARS.set(prefix, tree);
            });
        });
    },
    enhanceAppFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            let content = '';
            SIDEBARS.forEach((sidebar, prefix) => {
                content += `siteData.themeConfig.locales['${prefix}'].sidebar = ${JSON.stringify([sidebar])};\n`;
            });
            return {
                name: 'auto-sidebar-enhance-app',
                content: `export default ({ siteData }) => {
        ${content}
      }`,
            };
        });
    },
});
module.exports = GenerateTreePlugin;
//# sourceMappingURL=index.js.map